import 'package:dio/dio.dart';
import 'package:progamification/progamification.dart';
import 'package:progamification/src/environments/env.dart';
import 'package:progamification/src/services/auth.service.dart';
import 'package:shared_preferences/shared_preferences.dart';

class AuthInterceptor extends Interceptor {
  bool isRefreshing = false;

  Dio dio = Dio();

  @override
  Future<void> onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    String bearerToken = await getAuthToken();
    Map<String, String> requestHeaders = {
      'Content-type': 'application/json',
      'Accept': 'application/json',
      "Authorization": "Bearer $bearerToken",
    };
    options.headers.addAll(requestHeaders);
    return super.onRequest(options, handler);
  }

  @override
  Future<void> onError(DioError err, ErrorInterceptorHandler handler) async {
    // Handle the error based on the DioError instance
    if (err.response?.statusCode == 401 || err.response?.statusCode == 403) {
      if (!err.requestOptions.path.contains('accessToken')) {
        SharedPreferences preferences = await SharedPreferences.getInstance();
        String? refreshToken = preferences.getString('refreshToken');
        if (!isRefreshing && refreshToken != null) {
          isRefreshing = true;
          try {
            Response response =
                await GamificationAuthService.getAccessTokenByRefreshToken(
                    refreshToken);
            String accessToken = response.data['data']['token']['accessToken'];

            final retryOptions = Options(
              method: err.requestOptions.method,
              headers: {
                ...err.requestOptions.headers,
                'Authorization': 'Bearer $accessToken',
              },
              responseType: err.requestOptions.responseType,
            );

            final retryResponse = await dio.request(
              '$baseUrl${err.requestOptions.path}',
              options: retryOptions,
              data: err.requestOptions.data,
              queryParameters: err.requestOptions.queryParameters,
            );
            isRefreshing = false;
            handler.resolve(retryResponse);
            return;
          } catch (error) {
            isRefreshing = false;
            if (err.response?.statusCode == 401 ||
                err.response?.statusCode == 403) {
              Response response = await GamificationAuthService.init(
                  ProgamificationConfig.instance.clientId,
                  ProgamificationConfig.instance.clientSecret);
              String accessToken =
                  response.data['data']['tokens']['accessToken'];
              final retryOptions = Options(
                method: err.requestOptions.method,
                headers: {
                  ...err.requestOptions.headers,
                  'Authorization': 'Bearer $accessToken',
                },
                responseType: err.requestOptions.responseType,
              );

              final retryResponse = await dio.request(
                '$baseUrl${err.requestOptions.path}',
                options: retryOptions,
                data: err.requestOptions.data,
                queryParameters: err.requestOptions.queryParameters,
              );
              handler.resolve(retryResponse);
            }
          }
        }
      } else {
        handler.next(err);
      }
    }
  }

  static Future<String> getAuthToken() async {
    SharedPreferences preferences = await SharedPreferences.getInstance();
    return preferences.getString('gamificationAccessToken') ?? '';
  }
}
