import 'package:dio/dio.dart';
import 'package:progamification/src/constants/service_urls.dart';
import 'package:progamification/src/services/storage.service.dart';
import 'package:progamification/src/utilities/http_client.dart';
import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:shared_preferences/shared_preferences.dart';

class GamificationAuthService {
  static Future<dynamic> init(String? clientId, String? clientSecret) async {
    Dio httpClient = getHttpClient();
    Response response;
    try {
      response = await httpClient.get(
        ServiceUrls.getAuthTokens(clientId!, clientSecret!),
      );
      if (response.statusCode == 200) {
        dynamic data = response.data;
        String accessToken = data['data']['tokens']['accessToken'];
        String refreshToken = data['data']['tokens']['refreshToken'];
        await GamificationStorageService.setString(
            'gamificationAccessToken', accessToken);
        await GamificationStorageService.setString(
            'refreshToken', refreshToken);
      }
    } catch (exception) {
      throw Exception(exception);
    }
    return response;
  }

  static Future<dynamic> getAccessTokenByRefreshToken(
      String? refreshToken) async {
    Dio httpClient = getHttpClient();
    Map<String, dynamic> bodyMap = {'refreshToken': refreshToken};

    Response response;
    try {
      response = await httpClient.post(
        ServiceUrls.getAccessToken,
        data: bodyMap,
      );
      if (response.statusCode == 200) {
        dynamic data = response.data;
        String accessToken = data['data']['token']['accessToken'];
        await GamificationStorageService.setString(
            'gamificationAccessToken', accessToken);
      }
    } catch (exception) {
      throw Exception(exception);
    }

    return response;
  }

  static Future<bool> checkAccess() async {
    SharedPreferences preferences = await SharedPreferences.getInstance();
    String? accessToken = preferences.getString('gamificationAccessToken');
    String? refreshToken = preferences.getString('refreshToken');
    if (accessToken == null || refreshToken == null) {
      return false;
    }
    bool isExpired = JwtDecoder.isExpired(accessToken);
    if (isExpired) {
      return false;
    }
    return true;
  }
}
