import 'package:progamification/src/config/progamification.config.dart';
import 'package:progamification/src/controllers/leader_board.dart';
import 'package:progamification/src/services/auth.service.dart';
import 'package:progamification/src/services/leaderboard.service.dart';
import 'package:progamification/src/services/pro_gamfication.service.dart';
import 'package:progamification/src/services/sas_token.service.dart';
import 'package:progamification/src/services/storage.service.dart';
import 'package:progamification/src/services/user.service.dart';
import 'package:get/get.dart';
import 'package:progamification/src/models/user.dart';

class GamificationService extends GetxService {
  @override
  void onInit() async {
    super.onInit();
    await init();
  }

  static Future<void> init() async {
    Get.put<GamificationAuthService>(GamificationAuthService(),
        permanent: true);
    Get.put<GamificationStorageService>(GamificationStorageService(),
        permanent: true);
    Get.put<SasTokenService>(SasTokenService(), permanent: true);
    Get.put<LeaderBoardService>(LeaderBoardService(), permanent: true);
    Get.put<UserService>(UserService(), permanent: true);
    Get.put<LeaderBoardController>(LeaderBoardController(), permanent: true);
    GamificationStorageService.setString(
        'userId', ProgamificationConfig.instance.userId);
    GamificationStorageService.setString(
        'applicationId', ProgamificationConfig.instance.appId);
    bool access = await GamificationAuthService.checkAccess();

    if (!access) {
      await GamificationAuthService.init(
          ProgamificationConfig.instance.clientId,
          ProgamificationConfig.instance.clientSecret);
    }
  }

  static updateGameAction(String userId, String gameActionId,
      String correspondingUserId, String correspondingUserApplicationId) async {
    return await ProGamificationService().addPointsToUser({
      "userId": userId,
      "gameActionId": gameActionId,
      "correspondingUserId": correspondingUserId,
      "correspondingUserApplicationId": correspondingUserApplicationId
    });
  }

  static addUser(GamifiedUser user) async {
    return await UserService().addUser(user);
  }
}
