import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:progamification/src/widgets/avatar_placeholder.dart';
import 'package:progamification/src/widgets/bolt.dart';
import 'package:progamification/src/widgets/rank.dart';
import 'package:progamification/src/constants/color_contants.dart';
import 'package:progamification/src/controllers/leader_board.dart';
import 'package:progamification/src/models/leader_board.dart';
import 'package:progamification/src/services/sas_token.service.dart';
import 'package:progamification/src/services/storage.service.dart';
import 'package:scroll_to_index/scroll_to_index.dart';

class LeaderBoardTable extends StatefulWidget {
  const LeaderBoardTable({super.key});

  @override
  State<LeaderBoardTable> createState() => _LeaderBoardTableState();
}

class _LeaderBoardTableState extends State<LeaderBoardTable> {
  late AutoScrollController controller;

  LeaderBoardController leaderBoardController =
      Get.find<LeaderBoardController>();
  GamificationStorageService storageService =
      Get.find<GamificationStorageService>();
  LeaderBoardModel currentUser = LeaderBoardModel();
  SasTokenService sasTokenService = Get.find<SasTokenService>();
  String sasToken = "";
  String userId = '';
  @override
  initState() {
    super.initState();
    setCurrentUserLeaderBoardData();
    sasTokenService.getSasToken().then((value) {
      setState(() {
        sasToken = value.toString();
      });
    });
    storageService.getString('userId').then((value) => {
          setState(() {
            userId = value.toString();
          })
        });
    controller = AutoScrollController(
        viewportBoundaryGetter: () =>
            Rect.fromLTRB(0, 0, 0, MediaQuery.of(context).padding.bottom),
        axis: Axis.vertical);
  }

  TextStyle textStyle(fontSize, {Color color = Colors.black}) => TextStyle(
      color: color,
      fontWeight: FontWeight.w600,
      fontSize: fontSize,
      fontFamily: GoogleFonts.plusJakartaSans().fontFamily);
  @override
  Widget build(BuildContext context) {
    return SizedBox(
        height: MediaQuery.of(context).size.height,
        child: Scaffold(
            body: ListView.builder(
              controller: controller,
              shrinkWrap: true,
              itemCount: leaderBoardController.leaderBoardData.length,
              itemBuilder: (context, index) {
                Color rankColor = leaderBoardController
                            .leaderBoardData[index].userId
                            .toString() ==
                        userId
                    ? Colors.white
                    : Colors.black;
                return AutoScrollTag(
                    key: ValueKey(index),
                    controller: controller,
                    index: index,
                    child: Row(children: [
                      index == 0
                          ? SizedBox(
                              height: 220.0,
                              width: Get.width >= 600 ? 450 : Get.width,
                              child: const Rank(),
                            )
                          : Container(),
                      index >= 3
                          ? Container(
                              width: Get.width >= 600 ? 450 : Get.width,
                              decoration: BoxDecoration(
                                border: Border.all(
                                  color: leaderBoardController
                                              .leaderBoardData[index].userId
                                              .toString() ==
                                          userId
                                      ? ColorConstants.selectedTileColor
                                      : const Color.fromARGB(
                                          255, 238, 234, 234),
                                  width: 1,
                                ),
                                color: Colors.white,
                              ),
                              child: InkWell(
                                  onTap: () {
                                    // Handle tile tap
                                  },
                                  child: Padding(
                                      padding: const EdgeInsets.only(
                                          left: 30,
                                          right: 20,
                                          top: 8.0,
                                          bottom: 8.0),
                                      child: Row(children: [
                                        Container(
                                          decoration: BoxDecoration(
                                            shape: BoxShape.circle,
                                            color: leaderBoardController
                                                        .leaderBoardData[index]
                                                        .userId
                                                        .toString() ==
                                                    userId
                                                ? ColorConstants
                                                    .selectedTileColor
                                                : const Color(0xffEBEBEB),
                                          ),
                                          child: Padding(
                                            padding: const EdgeInsets.all(7.75),
                                            child: Text(
                                              leaderBoardController
                                                  .leaderBoardData[index].rank
                                                  .toString(),
                                              textAlign: TextAlign.start,
                                              style: textStyle(16.0,
                                                  color: rankColor),
                                            ),
                                          ),
                                        ),
                                        const SizedBox(width: 15),
                                        Container(
                                          width: 48.0,
                                          height: 48.0,
                                          decoration: const BoxDecoration(
                                            shape: BoxShape.circle,
                                          ),
                                          child: ClipOval(
                                            child: CachedNetworkImage(
                                              key: UniqueKey(),
                                              imageUrl: leaderBoardController
                                                      .leaderBoardData[index]
                                                      .profileImageUrl
                                                      .toString() +
                                                  sasToken,
                                              width: 50.0,
                                              height: 50.0,
                                              fit: BoxFit.fill,
                                              placeholder: (context, url) =>
                                                  const AvatarPlaceHolder(),
                                              errorWidget:
                                                  (context, url, error) =>
                                                      const AvatarPlaceHolder(),
                                            ),
                                          ),
                                        ),
                                        Padding(
                                          padding:
                                              const EdgeInsets.only(left: 8.0),
                                          child: SizedBox(
                                              width: 170,
                                              child: Text(
                                                leaderBoardController
                                                    .leaderBoardData[index]
                                                    .userName
                                                    .toString(),
                                                overflow: TextOverflow.ellipsis,
                                                softWrap: false,
                                                style: textStyle(16.0),
                                              )),
                                        ),
                                        Spacer(flex: 1),
                                        Row(
                                          children: [
                                            const Bolt(height: 55, width: 55),
                                            Text(
                                              textDirection: TextDirection.rtl,
                                              '${leaderBoardController.leaderBoardData[index].score}',
                                              style: textStyle(15.0),
                                            ),
                                          ],
                                        ),
                                      ]))),
                            )
                          : Container()
                    ]));
              },
            ),
            floatingActionButton: FloatingActionButton.extended(
              extendedPadding: const EdgeInsets.symmetric(horizontal: 0),
              backgroundColor: Colors.black,
              onPressed: () {
                int selectedIndex =
                    leaderBoardController.getCurrentUserIndex(userId)!;
                scrollToIndex(selectedIndex);
              },
              label: Row(children: [
                Container(
                    margin: const EdgeInsets.symmetric(horizontal: 10.0),
                    width: 33.0,
                    height: 33.0,
                    decoration: const BoxDecoration(
                      shape: BoxShape.circle,
                    ),
                    child: ClipOval(
                        child: CachedNetworkImage(
                      key: UniqueKey(),
                      imageUrl: currentUser.profileImageUrl + sasToken,
                      fit: BoxFit.fill,
                      placeholder: (context, url) => Image.asset(
                        "assets/icons/avatar.png",
                        package: "progamification",
                      ),
                      errorWidget: (context, url, error) => Image.asset(
                        "assets/icons/avatar.png",
                        package: "progamification",
                      ),
                    ))),
                Text("You're here",
                    style: TextStyle(
                        color: Colors.white,
                        fontFamily: GoogleFonts.plusJakartaSans().fontFamily)),
                const Icon(Icons.arrow_drop_down)
              ]),
            )));
  }

  Future<void> setCurrentUserLeaderBoardData() async {
    String userId = await storageService.getString('userId');
    LeaderBoardModel leaderboard =
        leaderBoardController.getLeaderBoardData(userId);
    setState(() {
      currentUser = leaderboard;
    });
  }

  Future scrollToIndex(int index) async {
    await controller.scrollToIndex(index,
        duration: const Duration(seconds: 1),
        preferPosition: AutoScrollPosition.middle);
  }
}
