import 'package:flutter/material.dart';

class LinearGradientProgressIndicator extends StatefulWidget {
  final double width;
  final double height;
  final double value;
  final Color backgroundColor;
  final List<Color> gradientColors;
  final Color borderColor;

  const LinearGradientProgressIndicator(
      {Key? key,
      this.width = double.infinity,
      this.height = 20.0,
      this.value = 0.0,
      this.backgroundColor = Colors.grey,
      required this.gradientColors,
      required this.borderColor})
      : super(key: key);

  @override
  State<LinearGradientProgressIndicator> createState() =>
      _LinearGradientProgressIndicatorState();
}

class _LinearGradientProgressIndicatorState
    extends State<LinearGradientProgressIndicator> {
  late LinearGradient _gradient;

  @override
  void initState() {
    super.initState();

    _gradient = LinearGradient(
      colors: widget.gradientColors,
      begin: Alignment.centerLeft,
      end: Alignment.centerRight,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      height: widget.height,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10.0),
        border: Border.all(color: widget.borderColor),
        color: widget.backgroundColor,
      ),
      child: ClipRRect(
        borderRadius: BorderRadius.circular(10.0),
        child: Stack(
          children: <Widget>[
            FractionallySizedBox(
                widthFactor: widget.value / 100,
                child: Container(
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(10.0),
                    gradient: _gradient,
                  ),
                )),
          ],
        ),
      ),
    );
  }
}
